package eu.mhsl.minenet.minigames.instance.game.stateless.types.towerdefense;

import eu.mhsl.minenet.minigames.instance.Dimension;
import eu.mhsl.minenet.minigames.instance.game.stateless.StatelessGame;
import eu.mhsl.minenet.minigames.instance.game.stateless.types.towerdefense.generator.MazeGenerator;
import eu.mhsl.minenet.minigames.instance.game.stateless.types.towerdefense.towers.BlazeTower;
import eu.mhsl.minenet.minigames.instance.game.stateless.types.towerdefense.towers.SkeletonTower;
import eu.mhsl.minenet.minigames.instance.game.stateless.types.towerdefense.towers.Tower;
import eu.mhsl.minenet.minigames.instance.game.stateless.types.towerdefense.towers.ZombieTower;
import eu.mhsl.minenet.minigames.score.LastWinsScore;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.entity.EntityType;
import net.minestom.server.entity.Player;
import net.minestom.server.instance.batch.AbsoluteBlockBatch;
import net.minestom.server.instance.block.Block;
import net.minestom.server.item.Material;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Random;

public class Towerdefense extends StatelessGame {
    private final Random random = new Random();
    private final AbsoluteBlockBatch mazeBatch = new AbsoluteBlockBatch();
    private final List<Pos> mazePath = new ArrayList<>();
    private final List<TowerdefenseRoom> instances = new ArrayList<>();
    private final Map<Material, Class<? extends Tower>> availableTowers = Map.of(
        Material.SKELETON_SPAWN_EGG, SkeletonTower.class,
        Material.ZOMBIE_SPAWN_EGG, ZombieTower.class,
        Material.BLAZE_SPAWN_EGG, BlazeTower.class
    );
    private final Map<Class<? extends Tower>, Integer> prices = Map.of(
        ZombieTower.class, 5,
        SkeletonTower.class, 10,
        BlazeTower.class, 15
    );

    private static final int pathLength = 10;

    public Towerdefense() {
        super(Dimension.NETHER.key, "Towerdefense", new LastWinsScore());

        this.setGenerator(new MazeGenerator());
        this.generateMaze();
    }

    private void generateMaze() {
        Pos position = new Pos(0, 0, 0);
        this.addMazePosition(position, Block.GREEN_WOOL);
        position = position.add(0,0,2);

        List<Integer> previousDirections = new ArrayList<>();
        int direction = 1;      // 0 -> right; 1 -> straight; 2 -> left
        for (int i = 0; i < pathLength; i++) {
            for (int j = 0; j < 9; j++) {
                position = position.add(direction-1,0,direction%2);
                this.addMazePosition(position, Block.WHITE_WOOL);
            }

            int origin = 0;
            int bound = 3;
            long rightLeftDifference = previousDirections.stream().filter(integer -> integer == 0).count() - previousDirections.stream().filter(integer -> integer == 2).count();
            if(rightLeftDifference >= 2 || direction == 2) origin = 1;
            if(rightLeftDifference <= -2 || direction == 0) bound = 2;
            direction = this.random.nextInt(origin, bound);
            previousDirections.add(direction);
        }
        this.addMazePosition(position, Block.WHITE_WOOL);
        this.addMazePosition(position.add(0,0,3), Block.WHITE_WOOL);
        this.addMazePosition(position.add(0,0,6), Block.RED_WOOL);
    }

    private void addMazePosition(Pos position, Block pathBlock) {
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                this.mazeBatch.setBlock(position.add(i-1,0,j-1), pathBlock);
            }
        }
        this.mazePath.add(position.add(0.5,1,0.5));
    }

    public AbsoluteBlockBatch getMazeBatch() {
        return this.mazeBatch;
    }

    public List<Pos> getMazePath() {
        return this.mazePath;
    }

    @Override
    protected boolean onPlayerJoin(Player p) {
        TowerdefenseRoom newRoom = new TowerdefenseRoom(p, this);
        this.instances.add(newRoom);
        p.setInstance(newRoom);
        newRoom.startWave(List.of(
            new GroupFactory(new EnemyFactory(EntityType.VILLAGER, 20, 0.1), 5, 800),
            new GroupFactory(new EnemyFactory(EntityType.BEE, 10, 1), 3, 400),
            new GroupFactory(new EnemyFactory(EntityType.SNIFFER, 1000, 0.02), 1, 0)
        ));
        return false;
    }

    public Map<Class<? extends Tower>, Integer> getPrices() {
        return this.prices;
    }

    public Map<Material, Class<? extends Tower>> getAvailableTowers() {
        return this.availableTowers;
    }
}
